/*
	Copyright 2007-2009 - Marc Dumoulin

	This file is part of Open-Maker software.

	Open-Maker software is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	Open-Maker software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/
#include "mainwindow.h"

MainWindow::MainWindow(QWidget* parent) : QMainWindow(parent) {
	QSettings settings(qApp->organizationName(),qApp->applicationName());
	setWindowIcon(QIcon(":/egg.png"));
	m_view = 0;
	m_active = false;
	m_timer = new QTimer;
	
	QFont font;
		font.setBold(true);
	
	m_numXLabel = new QLabel(tr("Width :"));
	m_numYLabel = new QLabel(tr("Height :"));
	m_numX = new QSpinBox;
		m_numX->setMinimum(3);
		if (settings.contains("width"))
			m_numX->setValue(settings.value("width").toInt());
		else m_numX->setValue(50);
	m_numY = new QSpinBox;
		m_numY->setMinimum(3);
		if (settings.contains("height"))
			m_numY->setValue(settings.value("height").toInt());
		else m_numY->setValue(30);
	m_reset = new QPushButton(tr("Reset with new size"));
		m_reset->setFont(font);
	m_state = new QPushButton(tr("Start"));
		m_state->setFont(font);
	m_next = new QPushButton(tr("Next Turn"));
		m_next->setFont(font);
	m_clear = new QPushButton(tr("Clear"));
		m_clear->setFont(font);
	m_speed = new QSlider(Qt::Horizontal);
		m_speed->setMaximum(5);
		m_speed->setTickPosition(QSlider::TicksBothSides);
		if (settings.contains("speed"))
			m_speed->setValue(settings.value("speed").toInt());
		else m_speed->setValue(5);
	m_speedLabel = new QLabel(tr("Speed :"));
	
	QMenuBar* bar = new QMenuBar;
		QMenu* file = new QMenu(tr("File"));
			file->addAction(tr("Open pattern..."),this,SLOT(openFile()),QKeySequence::Open);
			file->addAction(tr("Save pattern..."),this,SLOT(saveFile()),QKeySequence::Save);
			file->addSeparator();
			file->addAction(tr("Close"),qApp,SLOT(quit()),QKeySequence::Close);
		QMenu* colors = new QMenu(tr("Colors"));
			colors->addAction(tr("Background color..."),this,SLOT(callNAColor()));
			colors->addAction(tr("Active color..."),this,SLOT(callAColor()));
			colors->addSeparator();
			colors->addAction(tr("Set colors to default"),this,SLOT(callDefColor()));
		QMenu* set = new QMenu(tr("Settings"));
			QAction* aset = set->addAction(tr("Remember settings"));
				aset->setCheckable(true);
				aset->setChecked(settings.contains("remember"));
			connect(set,SIGNAL(triggered(QAction*)),this,SLOT(rememberSettings(QAction*)));
		QMenu* etc = new QMenu(tr("?"));
			etc->addAction(tr("About QLife..."),this,SLOT(about()));
			etc->addAction(tr("About Qt..."),qApp,SLOT(aboutQt()));
		bar->addMenu(file);
		bar->addMenu(colors);
		bar->addMenu(set);
		bar->addMenu(etc);
	setMenuBar(bar);
	
	reset(true);
}

void MainWindow::openFile() {
	QString path = QFileDialog::getOpenFileName();
	if (path.isEmpty()) return;
	QFile file(path);
	if (!file.open(QIODevice::ReadOnly)) {
		QMessageBox::critical(0,tr("Error"),tr("Could not open the specified file."));
		return;
	}
	QTextStream in(&file);
	if (!in.atEnd()) {
		QStringList list = in.readLine().split(" ");
		if (list.size() < 2) {
			QMessageBox::critical(0,tr("Error"),tr("The informations in the file could not be read correctly."));
			return;
		}
		m_numX->setValue(list[0].toInt());
		m_numY->setValue(list[1].toInt());
		reset();
	}
	int y = 0;
	while (!in.atEnd() && y < m_numY->value()) {
		QString line = in.readLine();
		for (int x = 0; x < line.size(); x++) {
			if (x == m_numX->value())
				break;
			m_view->setState(x,y,(line[x].digitValue() == 0? false : true));
		}
		y++;
	}
	m_timer->stop();
	m_active = false;
}

void MainWindow::saveFile() {
	QString path = QFileDialog::getSaveFileName();
	if (path.isEmpty()) return;
	QFile file(path);
	if (!file.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
		QMessageBox::critical(0,tr("Error"),tr("Could not open the specified file."));
		return;
	}
	QTextStream out(&file);
	out << m_view->width() << " " << m_view->height() << endl;
	for (int y = 0; y < m_view->height(); y++) {
		for (int x = 0; x < m_view->width(); x++)
			out << (m_view->state()[x][y]? 1 : 0);
		out << endl;
	}
}

void MainWindow::rememberSettings(QAction* a) {
	QSettings settings(qApp->organizationName(),qApp->applicationName());
	if (!a->isChecked())
		return settings.clear();
	settings.setValue("remember",true);
	saveSettings();
}

void MainWindow::saveSettings() {
	QSettings settings(qApp->organizationName(),qApp->applicationName());
	if (!settings.contains("remember")) return;
	settings.setValue("a_color",m_view->activeColor());
	settings.setValue("na_color",m_view->inactiveColor());
	settings.setValue("speed",m_speed->value());
	settings.setValue("width",m_numX->value());
	settings.setValue("height",m_numY->value());
}

void MainWindow::reset(bool isAtStartup) {
	if (!isAtStartup && m_numX->value() == m_view->width() && m_numY->value() == m_view->height())
		return;
	if (m_view != 0)
		delete m_view;
	m_view = new QLifeGraphicsView(m_numX->value(),m_numY->value(),10,10);
	if (isAtStartup) {
		connect(m_reset,SIGNAL(clicked()),this,SLOT(reset()));
		connect(m_state,SIGNAL(clicked()),this,SLOT(stateChange()));
		connect(m_next,SIGNAL(clicked()),this,SLOT(nextTurn()));
		connect(m_timer,SIGNAL(timeout()),this,SLOT(nextTurn()));
		connect(m_speed,SIGNAL(valueChanged(int)),this,SLOT(setInterval(int)));
	}
	connect(m_clear,SIGNAL(clicked()),m_view,SLOT(clear()));
	m_timer->stop();
	m_active = false;
	m_state->setText(tr("Start"));
	
	QVBoxLayout* mainLayout = new QVBoxLayout;
		QHBoxLayout* sizeLayout = new QHBoxLayout;
			sizeLayout->addWidget(m_numXLabel);
			sizeLayout->addWidget(m_numX);
			sizeLayout->addWidget(m_numYLabel);
			sizeLayout->addWidget(m_numY);
			sizeLayout->addWidget(m_reset);
			sizeLayout->addStretch(1);
		mainLayout->addLayout(sizeLayout);
		QHBoxLayout* subMainLayout = new QHBoxLayout;
			subMainLayout->addWidget(m_view);
			QVBoxLayout* buttonLayout = new QVBoxLayout;
				buttonLayout->addWidget(m_state);
				buttonLayout->addWidget(m_next);
				buttonLayout->addStretch(1);
				buttonLayout->addWidget(m_speedLabel);
				buttonLayout->addWidget(m_speed);
				buttonLayout->addStretch(1);
				buttonLayout->addWidget(m_clear);
			subMainLayout->addLayout(buttonLayout);
		mainLayout->addLayout(subMainLayout);
	QWidget* dummy = new QWidget;
		dummy->setLayout(mainLayout);
	setCentralWidget(dummy);
	
	QTimer::singleShot(500,this,SLOT(adjustToMinimalSize()));
	saveSettings();
}

void MainWindow::callNAColor() {
	if (m_active) m_timer->stop();
	m_view->newInactiveColor();
	if (m_active) m_timer->start();
	saveSettings();
}

void MainWindow::callAColor() {
	if (m_active) m_timer->stop();
	m_view->newActiveColor();
	if (m_active) m_timer->start();
	saveSettings();
}

void MainWindow::callDefColor() {
	if (m_active) m_timer->stop();
	m_view->setDefaultColor();
	if (m_active) m_timer->start();
	saveSettings();
}

void MainWindow::adjustToMinimalSize() {
	resize(1,1);
}

void MainWindow::setInterval(int value) {
	m_timer->setInterval(1000-((value+1)*150));
	saveSettings();
}

void MainWindow::stateChange() {
	m_active = !m_active;
	m_timer->setInterval(1000-((m_speed->value()+1)*150));
	if (m_active) {
		m_state->setText(tr("Stop"));
		m_timer->start();
	}
	else {
		m_state->setText(tr("Start"));
		m_timer->stop();
	}
}

void MainWindow::nextTurn() {
	QList<QList<bool> > nextState = m_view->state();
	for (int y = 0; y < m_view->height(); y++)
		for (int x = 0; x < m_view->width(); x++) {
			int n = numberOfNeighboor(x,y,m_view->state());
			if (n < 2)
				nextState[x][y] = false;
			if (n == 3)
				nextState[x][y] = true;
			if (n > 3)
				nextState[x][y] = false;
		}
	for (int y = 0; y < m_view->height(); y++)
		for (int x = 0; x < m_view->width(); x++) {
			if (nextState[x][y] != m_view->state()[x][y])	
				m_view->setState(x,y,nextState[x][y]);
		}
}

int MainWindow::numberOfNeighboor(int rx,int ry,const QList<QList<bool> >& state) {
	int n = 0;
	for (int y = ry-1; y < ry+2; y++)
		for (int x = rx-1; x < rx+2; x++) {
			if (x == rx && y == ry)
				continue;
			if (x >= 0 && x < m_view->width() && y >= 0 && y < m_view->height())
				if (state[x][y])
					n++;
		}
	return n;
}

void MainWindow::about() {
	QMessageBox::about(this,tr("About %1...").arg(qApp->applicationName()),
							tr("<b>Author</b> : Marc Dumoulin"
							"<br/>"
							"<b>Version</b> : %1"
							"<br/>"
							"<b>Licence</b> : GPL v3"
							"<br/>"
							"<b>Creation</b> : %2"
							"<br/>"
							"<b>Site web</b> : <a href='%3'>%4</a>").
							arg(
								qApp->applicationVersion(),
								"17/03/2009",
								qApp->organizationDomain(),
								qApp->organizationName())
							);
}
