/*
	Copyright 2007-2009 - Marc Dumoulin

	This file is part of Open-Maker software.

	Open-Maker software is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	Open-Maker software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/
#ifndef QLIFEGRAPHICSVIEW_H
#define QLIFEGRAPHICSVIEW_H

#include <QGraphicsView>
#include <QGraphicsScene>
#include <QMouseEvent>
#include <QGraphicsRectItem>
#include <QColorDialog>
#include <QSettings>
#include <QApplication>

class QLifeGraphicsView : public QGraphicsView {
	Q_OBJECT	
	public :
		QLifeGraphicsView(int amountWidth,int amountHeight,int width,int height);
		~QLifeGraphicsView();
		
		QList<QList<bool> >& state();
		void setState(int x,int y,bool s);
		int width();
		int height();
		QColor activeColor();
		QColor inactiveColor();
		void newActiveColor();
		void newInactiveColor();
		void setDefaultColor();
	public slots :
		void clear();
	protected :
		void mouseMoveEvent(QMouseEvent* event);
		void mousePressEvent(QMouseEvent* event);
		void mouseReleaseEvent(QMouseEvent* event);
	private :
		QGraphicsRectItem* getNode(const QPoint& pos);
		bool* getNodeState(const QPoint& pos);
	private :
		QGraphicsScene* m_scene;
		QList<QList<QGraphicsRectItem*> > m_grid;
		QList<QList<bool> > m_state;
		
		bool m_leftMousePressed;
		bool m_rightMousePressed;
		
		int m_amountWidth;
		int m_amountHeight;
		int m_width;
		int m_height;
		
		QColor m_activeColor;
		QColor m_inactiveColor;
};

#endif
